﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Text;

namespace Dreadmill
{
    static class DreadmillDriver
    {
        public static List<string> errorLog = new List<string>();

        static SerialPort _Serial = null;
        static bool running = false;
        static byte input = 16;

        static public void Initialise()
        {
            string[] ports = SerialPort.GetPortNames();
            if (ports.Length > 0)
            {
                errorLog.Add("Connecting to port: " + ports[ports.Length -1]);
                _Serial = new SerialPort(ports[ports.Length -1]);
                _Serial.DataReceived += SerialDataReceived;
                _Serial.BaudRate = 9600;
                _Serial.NewLine = "\n";
                
                try
                {
                    _Serial.Open();
                    CheckElevation();
                    
                    SetSpeed("150");
                    SetHeight(250);
                }
                catch (Exception ex)
                {
                    Console.WriteLine("Com Port Error!");
                    errorLog.Add("Com Port Found!");
                    _Serial = null;
                }
            }
            else
            {
                errorLog.Add("No Com Port Found!");
            }
        }

        static public void CheckElevation()
        {
            if (_Serial == null || !_Serial.IsOpen)
                {Console.WriteLine("No Serial Port"); return;}
            _Serial.WriteLine("CE");
        }

        static public void SetHeight(int height)
        {
            if (_Serial == null || !_Serial.IsOpen)
                {Console.WriteLine("No Serial Port"); return;}
            _Serial.WriteLine("SH" + height);
        }

        static public void SetSpeed(string speed)
        {
            if (_Serial == null || !_Serial.IsOpen)
                {Console.WriteLine("No Serial Port"); return;}
            int speedInt = Convert.ToInt32(speed);
            if (speedInt < 50 || speedInt > 200)
                return;
            _Serial.WriteLine("PI" + speed);
        }

        static public void SetElevation(string elevation)
        {
            if (_Serial == null || !_Serial.IsOpen)
                {Console.WriteLine("No Serial Port"); return;}
            _Serial.WriteLine("SH" + elevation);
        }
        static public void Start()
        {
            if (_Serial == null || !_Serial.IsOpen)
                {Console.WriteLine("No Serial Port"); return;}
            _Serial.WriteLine("GO");
            running = true;
        }

        static public void Stop()
        {
            if (_Serial == null || !_Serial.IsOpen)
                {Console.WriteLine("No Serial Port"); return;}
            _Serial.WriteLine("ST");
            running = false;
        }

        static public void Shutdown()
        {
            if (_Serial != null && _Serial.IsOpen)
            {
                _Serial.WriteLine("ST");
                _Serial.Close();
            }
            running = false;
        }

        static public bool IsRunning()
        {
            if (_Serial == null || !_Serial.IsOpen)
                return false;
            return running;
        }

        static public byte GetInput()
        {
            return input;
        }

        static private void SerialDataReceived(object sender, SerialDataReceivedEventArgs args)
        {
            string Data = ((SerialPort)sender).ReadLine();
            Console.WriteLine("Data Recieved: " + Data);
            errorLog.Add("Data Recieved: " + Data);
            if (Data.IndexOf("IN:") == 0)
            {
                byte newInput;
                if (Byte.TryParse(Data.Substring(3), out newInput))
                {
                    DreadmillDriver.input = newInput;
                    DreadMill.UpdateInput(newInput); // Sends to Server from Client
                }
                else
                {
                    Console.WriteLine("Unpassable Input: " + Data);
                    errorLog.Add("Unpassable Input: " + Data);

                }

               
            }
        }
        
    }
}
