﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using Lidgren.Network;
using Microsoft.Xna.Framework;

namespace Dreadmill
{
    struct dataPacket
    {
        public Vector2 position;
        public Vector2 velocity;
    }

    static class DreadmillNetwork
    {
        static private NetServer server;
        static bool isServer = false;
        static bool connected = false;
        static private NetConnection remoteHost;

        static Lidgren.Network.NetClient client;

        static public dataPacket remote, local;
        static private dataPacket lastLocal;

        static public bool VUpdate = false;
        static public bool PUpdate = false;

        static public byte remoteInput = 16;

        static public bool player1ready = false;
        static public bool player2ready = false;

        static public bool gameOver = false;
        static public string winner = "";

        static public Queue<string> items = new Queue<string>();

        static void Start()
        {

        }

        static void FindServer(int timeout)
        {

        }

        static public byte GetClientInput()
        {
            return remoteInput;
        }

        static public void StartClient()
        {
            NetPeerConfiguration config = new NetPeerConfiguration("Dreadmill");
            config.EnableMessageType(NetIncomingMessageType.DiscoveryRequest);
            config.EnableMessageType(NetIncomingMessageType.DiscoveryResponse);

            client = new NetClient(config);
            //search in local network at port 50001
            client.Start();
        }

        static public void StopClient()
        {
            if (client != null)
                client.Disconnect("");
            client = null;
        }

        static public void ConnectToIP(string IP)
        {
            //client.DiscoverLocalPeers(50001);
            remoteHost = client.Connect(IP, 50001);

        }

        static public bool CheckClientConnection()
        {
            if (remoteHost.Status == NetConnectionStatus.Connected)
                return true;
            return false;
        }
        static public bool FindServer()
        {
            NetPeerConfiguration config = new NetPeerConfiguration("Dreadmill");
            config.EnableMessageType(NetIncomingMessageType.DiscoveryRequest);
            config.EnableMessageType(NetIncomingMessageType.DiscoveryResponse);

            client = new NetClient(config);
            //search in local network at port 50001
            client.Start();

            client.DiscoverLocalPeers(50001);

            System.Threading.Thread.Sleep(1000);

            //Check For Responses
            NetIncomingMessage inc;
            while ((inc = client.ReadMessage()) != null)
            {
                switch (inc.MessageType)
                {
                    case NetIncomingMessageType.DiscoveryResponse:
                        string hostName = inc.ReadString();
                        Console.WriteLine("Found server at " + inc.SenderEndPoint + " name: " + hostName);
                        if (client.ServerConnection != null)
                            Console.WriteLine("No Server Connection");


                        NetConnection gameServer = client.Connect(hostName, 50001);

                        if (gameServer.Status == NetConnectionStatus.Connected)
                        {
                            Console.WriteLine("Great Success");
                            return true;
                        }
                        break;

                }
            }
            return false;
        }

        static public void StartBroadcastServer()
        {
            isServer = true;
            NetPeerConfiguration config = new NetPeerConfiguration("Dreadmill");
            //Setting the port, where the NetPeer shall listen. 
            //This is optional, but for a server its good to know, where it is reachable
            config.Port = 50001;
            //config.EnableMessageType(NetIncomingMessageType.ConnectionApproval);
            //config.EnableMessageType(NetIncomingMessageType.DiscoveryRequest);
            // Enable DiscoveryResponse messages
            //config.EnableMessageType(NetIncomingMessageType.DiscoveryResponse);
            //config.LocalAddress = NetUtility.Resolve("10.5.2.207");

            //Create the NetPeer object with the configurations.
            server = new NetServer(config);
            //Start
            //for the server
            server.Start();

        }

        static public void SendCommandToClient(String command)
        {
            if (isServer)
            {
                if (connected)
                {
                    if (server.Connections.Count > 0)
                    {
                        NetOutgoingMessage sendMsg = server.CreateMessage();
                        sendMsg.Write(command);
                        server.SendMessage(sendMsg, server.Connections[0], NetDeliveryMethod.ReliableOrdered);
                    }
                }

            }
        }

        static public void SendCommandToServer(String command)
        {
            if (!isServer)
            {
                if (remoteHost != null)
                    if (remoteHost.Status == NetConnectionStatus.Connected)
                    {
                        NetOutgoingMessage sendMsg = client.CreateMessage();
                        sendMsg.Write(command);
                        client.SendMessage(sendMsg, remoteHost, NetDeliveryMethod.ReliableOrdered);
                    }
            }
        }

        static public void Update()
        {
            if (isServer)
            {
                UpdateServer();
            }
            else // Client connected via remoteHost connection
            {
                UpdateClient();
            }
        }

        static public void UpdateServer()
        {
            NetIncomingMessage msg;
            while ((msg = server.ReadMessage()) != null)
            {
                switch (msg.MessageType)
                {
                    case NetIncomingMessageType.DiscoveryRequest:

                        // Create a response and write some example data to it
                        NetOutgoingMessage response = server.CreateMessage();
                        response.Write(Dns.GetHostName());

                        // Send the response to the sender of the request
                        server.SendDiscoveryResponse(response, msg.SenderEndPoint);
                        break;
                    case NetIncomingMessageType.VerboseDebugMessage:
                    case NetIncomingMessageType.DebugMessage:
                    case NetIncomingMessageType.WarningMessage:
                    case NetIncomingMessageType.ErrorMessage:
                        Console.WriteLine(msg.ReadString());
                        break;
                    case NetIncomingMessageType.StatusChanged:
                        NetConnectionStatus status = (NetConnectionStatus)msg.ReadByte();
                        Console.WriteLine("Status Message: " + msg.ReadString());

                        if (status == NetConnectionStatus.Connected)
                        {
                            //PLAYER CONNECTED
                            Console.WriteLine("Let the Game Begin!!!");
                            connected = true;
                            break;
                        }
                        break;
                    case NetIncomingMessageType.Data:
                        // Process Vector Data
                        string command = msg.ReadString();

                        Console.WriteLine("Recieved Packet: " + command);
                        if (command == "P")
                        {
                            float px = msg.ReadFloat();
                            float py = msg.ReadFloat();
                            remote.position.X = px;
                            remote.position.Y = py;
                            PUpdate = true;
                        }
                        else if (command == "V")
                        {
                            float px = msg.ReadFloat();
                            float py = msg.ReadFloat();
                            remote.velocity.X = px;
                            remote.velocity.Y = py;
                            VUpdate = true;
                        }
                        else if (command == "PV")
                        {
                            float px = msg.ReadFloat();
                            float py = msg.ReadFloat();
                            remote.position.X = px;
                            remote.position.Y = py;
                            px = msg.ReadFloat();
                            py = msg.ReadFloat();
                            remote.velocity.X = px;
                            remote.velocity.Y = py;
                            PUpdate = true;
                            VUpdate = true;
                        }
                        else if (command.StartsWith("IN:"))
                        {
                            remoteInput = Convert.ToByte(command.Substring(3));
                            Console.WriteLine("Client Input Updated to:" + remoteInput);
                        }
                        break;
                    default:
                        Console.WriteLine("Unhandled type: " + msg.MessageType);
                        break;
                }
                server.Recycle(msg);
            }

            if (connected)
            {
                if (server.Connections.Count > 0)
                {
                    if ((lastLocal.velocity == local.velocity && local.velocity != Vector2.Zero) ||
                        (local.velocity == Vector2.Zero && lastLocal.position == local.position))
                    {
                        Console.WriteLine("Skipping Send, No Packet Change");
                    }
                    else
                    {
                        Console.WriteLine("Sending Packet");
                        NetOutgoingMessage sendMsg = server.CreateMessage();
                        if (lastLocal.velocity != local.velocity && local.velocity == Vector2.Zero) // lastLocal.position != local.position && 
                        {
                            //PV
                            sendMsg.Write("PV");
                            sendMsg.Write(local.position.X);
                            sendMsg.Write(local.position.Y);
                            sendMsg.Write(local.velocity.X);
                            sendMsg.Write(local.velocity.Y);
                        }
                        /*else if (lastLocal.position != local.position)
                        {
                            // P
                            sendMsg.Write("P");
                            sendMsg.Write(local.position.X);
                            sendMsg.Write(local.position.Y);
                        }*/
                        else
                        {
                            sendMsg.Write("V");
                            sendMsg.Write(local.velocity.X);
                            sendMsg.Write(local.velocity.Y);
                        }
                        server.SendMessage(sendMsg, server.Connections[0], NetDeliveryMethod.ReliableOrdered);
                        lastLocal = local;
                    }
                }
                else
                {
                    connected = false;
                    Console.WriteLine("Client Lost!");
                }
            }
        }

        static public void UpdateClient()
        {
            if ((lastLocal.velocity == local.velocity && local.velocity != Vector2.Zero) ||
                (local.velocity == Vector2.Zero && lastLocal.position == local.position))
            {
                //  Console.WriteLine("Skipping Send, No Packet Change");
            }
            else
            {
                NetOutgoingMessage sendMsg = client.CreateMessage();
                if (lastLocal.velocity != local.velocity && local.velocity == Vector2.Zero) // lastLocal.position != local.position && 
                {
                    //PV
                    sendMsg.Write("PV");
                    sendMsg.Write(local.position.X);
                    sendMsg.Write(local.position.Y);
                    sendMsg.Write(local.velocity.X);
                    sendMsg.Write(local.velocity.Y);
                }
                /*else if (lastLocal.position != local.position)
                {
                    // P
                    sendMsg.Write("P");
                    sendMsg.Write(local.position.X);
                    sendMsg.Write(local.position.Y);
                }*/
                else
                {
                    sendMsg.Write("V");
                    sendMsg.Write(local.velocity.X);
                    sendMsg.Write(local.velocity.Y);
                }
                // Console.WriteLine("Sending Packet to Server");

                client.SendMessage(sendMsg, remoteHost, NetDeliveryMethod.ReliableOrdered);
                lastLocal = local;
            }
            NetIncomingMessage msg;
            while ((msg = client.ReadMessage()) != null)
            {
                switch (msg.MessageType)
                {
                    case NetIncomingMessageType.Data:
                        // Process Vector Data
                        string command = msg.ReadString();

                        Console.WriteLine("Recieved Packet: " + command);
                        if (command == "P")
                        {
                            float px = msg.ReadFloat();
                            float py = msg.ReadFloat();
                            remote.position.X = px;
                            remote.position.Y = py;
                            PUpdate = true;
                        }
                        else if (command == "V")
                        {
                            float px = msg.ReadFloat();
                            float py = msg.ReadFloat();
                            remote.velocity.X = px;
                            remote.velocity.Y = py;
                            VUpdate = true;
                        }
                        else if (command == "PV")
                        {
                            float px = msg.ReadFloat();
                            float py = msg.ReadFloat();
                            remote.position.X = px;
                            remote.position.Y = py;
                            px = msg.ReadFloat();
                            py = msg.ReadFloat();
                            remote.velocity.X = px;
                            remote.velocity.Y = py;
                            PUpdate = true;
                            VUpdate = true;
                        }
                        else if (command == "ST")
                        {
                            DreadmillDriver.Stop();
                        }
                        else if (command == "GO")
                        {
                            DreadmillDriver.Start();
                        }
                        else if (command.IndexOf("PI") == 0)
                        {
                            DreadmillDriver.SetSpeed(command.Substring(2));
                        }
                        else if (command.IndexOf("SH") == 0)
                        {
                            DreadmillDriver.SetElevation(command.Substring(2));
                        }
                        else if (command.IndexOf("GAME") == 0)
                        {
                            player2ready = true;
                            player1ready = true;
                        }
                        else if (command.IndexOf("END") == 0)
                        {
                            gameOver = true;
                            winner = command;
                        }
                        else if (command.IndexOf("ZOMBIE") == 0)
                        {
                            items.Enqueue(command);
                        }
                        else if (command.IndexOf("POWER") == 0)
                        {
                            items.Enqueue(command);
                        }
                        break;
                    default:
                        Console.WriteLine("Unhandled type: " + msg.MessageType);
                        break;
                }
            }
        }
    }
}
