using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Dreadmill
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        public string debugText = "DEBUG";
        public SpriteFont debugFont;
        public SpriteFont menuFont;

        public GraphicsDeviceManager graphics;
        public SpriteBatch spriteBatch;
        public GameComponentCollection components;

        TitleState title;
        public PlayState playState;
        GameOverState gameOverState;
        NetworkSearchState networkSearch;

        KeyboardState previousKeyboard;
        public float roadSpeed = 4.0f;


        public SoundEngineXACT soundEngine;

        public enum GameState
        {
            Title,
            Gameplay,
            NetworkSearch,
            GameOver
        }

        GameState state;

        public Random rand = new Random();

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 1024;
            graphics.PreferredBackBufferHeight = 720;
            graphics.ApplyChanges();
           //  graphics.IsFullScreen = true;
            Content.RootDirectory = "Content";

            title = new TitleState(this, graphics, Content, components);
            playState = new PlayState(this, graphics, Content, components);
            gameOverState = new GameOverState(this, graphics, Content, components);
            networkSearch = new NetworkSearchState(this, graphics, Content, components);

            previousKeyboard = Keyboard.GetState();
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            state = GameState.Title;
            title.Initialize();
            playState.Initialize();
            gameOverState.Initialize();
            networkSearch.Initialize();

            soundEngine = new SoundEngineXACT(this);

            Components.Add(soundEngine);

            
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            debugFont = Content.Load<SpriteFont>("debug");
            menuFont = Content.Load<SpriteFont>("menuLarge");
            // TODO: use this.Content to load your game content here

            

            soundEngine.PlaySong("Fake");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            switch (state)
            {
                case GameState.Title:
                    title.Update(gameTime);
                    break;
                case GameState.NetworkSearch:
                    networkSearch.Update(gameTime);
                    if (networkSearch.searchExpired  && networkSearch.PlayersAccepted())
                    {
                        state = GameState.Gameplay;
                        playState.InitialiseState();
                        //networkSearch = new NetworkSearchState(this, graphics, Content, components);
                        //networkSearch.Initialize();
                    }
                    break;
                case GameState.Gameplay:
                    playState.Update(gameTime);
                    break;
                case GameState.GameOver:
                    gameOverState.Update(gameTime);
                    break;
                default:
                    break;
            }

            // Allows the game to exit
            if(Keyboard.GetState().IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }

            if (Keyboard.GetState().IsKeyDown(Keys.Space) && previousKeyboard.IsKeyUp(Keys.Space))
            {
                switch (state)
                {
                    case GameState.Title:
                        state = GameState.NetworkSearch;
                        soundEngine.PlayBackground("Laugh");
                        break;
                    case GameState.NetworkSearch:
                        //state = GameState.Gameplay;
                        break;
                    case GameState.Gameplay:
                        state = GameState.GameOver;
                        break;
                    case GameState.GameOver:
                        state = GameState.Title;
                        break;
                    default:
                        break;
                }
            }

            previousKeyboard = Keyboard.GetState();

            // TODO: Add your update logic here

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            switch (state)
            {
                case GameState.Title:
                    title.Draw(gameTime);
                    break;
                case GameState.NetworkSearch:
                    title.Draw(gameTime);
                    networkSearch.Draw(gameTime);
                    break;
                case GameState.Gameplay:
                    playState.Draw(gameTime);
                    break;
                case GameState.GameOver:
                    gameOverState.Draw(gameTime);
                    break;
                default:
                    break;
            }

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }

        public void gameOver(Player loser)
        {
            state = GameState.GameOver;
            gameOverState.assignLoser(loser.name);
        }
    }
}
