using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;


namespace Dreadmill
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class NetworkSearchState : Microsoft.Xna.Framework.DrawableGameComponent
    {
        Game1 game;
        GraphicsDeviceManager graphics;
        ContentManager content;
        GameComponentCollection components;

        int searchTimeout;
        bool timedOut;
        public bool searchExpired;
        public bool serverFound;
        bool clientStarted = false;

        bool allConnected = true;

        public int player1Accepted = 0;
        public int player2Accepted = 0;

        Texture2D titleTex;
        Texture2D warning1, warning2;

        DateTime nextSearch = DateTime.Now;
        DateTime endSearch = DateTime.Now;
        

        public NetworkSearchState(Game1 game, GraphicsDeviceManager graphics, ContentManager content, GameComponentCollection components)
            : base(game)
        {
            // TODO: Construct any child components here
            this.graphics = graphics;
            this.content = content;
            this.components = components;
            this.game = game;

            searchTimeout = 5;
            timedOut = false;
            searchExpired = false;
            serverFound = false;
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here
            titleTex = game.Content.Load<Texture2D>("Title");

            warning1 = game.Content.Load<Texture2D>("Warning");
            warning2 = game.Content.Load<Texture2D>("Warning1");
            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            if (!searchExpired)
            {
                if (!clientStarted)
                {
                    DreadmillNetwork.StartClient();
                    DreadmillNetwork.ConnectToIP("10.5.2.207"); // 207 190
                    clientStarted = true;
                    endSearch = DateTime.Now.AddSeconds(searchTimeout);
                }
                // TODO: Add your update code here
                if (DateTime.Now < endSearch && !serverFound)
                {
                    if (DateTime.Now > nextSearch)
                    {
                        DreadmillNetwork.Update();
                        nextSearch = nextSearch.AddSeconds(0.5f);
                    }

                    //System.Threading.Thread.Sleep(1000);

                    if (DreadmillNetwork.CheckClientConnection() == true) //207 = BlackMac 190 = Kimmii
                    {
                        serverFound = true;
                        Console.WriteLine("Found Server!");
                        DreadMill.SetMode(false);
                        ((Game1)game).debugText = "DEBUG CLIENT";
                    }
                }
                else //if(!serverFound)
                {
                    timedOut = true;
                    DreadMill.SetMode(true);
                    ((Game1)game).debugText = "DEBUG SERVER";
                    DreadmillNetwork.StopClient();
                    Console.WriteLine("Starting Broadcast Server");
                    DreadmillNetwork.StartBroadcastServer();
                }

                if (timedOut || serverFound)
                {
                    searchExpired = true;
                }
            }
            else
            {
                DreadmillNetwork.Update();


                byte player1Input = 0;
                byte player2Input = 0;
                if (DreadMill.IsGameServer())
                {
                    player1Input = DreadmillDriver.GetInput();
                    player2Input = DreadmillNetwork.GetClientInput();
                    if (IsDown(player1Input, 5) && IsDown(player2Input, 5))
                    {
                        allConnected = true;

                    }
                    else
                        allConnected = false;
                }
                else
                {
                    player2Input = DreadmillDriver.GetInput();
                    if (IsDown(player2Input, 5))
                        allConnected = true;
                    else
                        allConnected = false;
                }

                //Process Player Input
                if ((IsDown(player1Input, 3) || Keyboard.GetState().IsKeyDown(Keys.Left)) && player1Accepted == 0)
                {
                    player1Accepted++;
                }
                else if ((IsDown(player1Input, 1) || Keyboard.GetState().IsKeyDown(Keys.Right)) && player1Accepted == 1)
                {
                    player1Accepted++;
                }
                //Process Player Input
                if ((IsDown(player2Input, 3) || Keyboard.GetState().IsKeyDown(Keys.A)) && player2Accepted == 0)
                {
                    player2Accepted++;
                }
                else if ((IsDown(player2Input, 1) || Keyboard.GetState().IsKeyDown(Keys.D)) && player2Accepted == 1)
                {
                    player2Accepted++;
                }

                bool test = PlayersAccepted();

                if (Keyboard.GetState().IsKeyDown(Keys.Space))
                {
                    player1Accepted = 2;
                    player2Accepted = 2;
                    allConnected = true;
                }
            }


            base.Update(gameTime);
        }

        public bool IsDown(byte input, int button)
        {
            int data = input >> button - 1;
            data = data & 0x01;
            if (data == 0)
                return false;
            return true;
        }


        public bool PlayersAccepted()
        {
            if (DreadMill.IsGameServer())
            {
                if (player1Accepted > 1 && player2Accepted > 1)
                {
                    // Send Start Message to Remote Host
                    DreadmillNetwork.SendCommandToClient("GAME");
                    return true;
                }
                return false;
            }
            else // Client
            {
                if (DreadmillNetwork.player1ready && DreadmillNetwork.player2ready)
                {
                    return true;
                }
                return false;
            }
        }

        public override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Magenta);
            if (!allConnected)
            {
                GraphicsDevice.Clear(Color.Red);
            }
            // TODO: Add your drawing code here
            game.spriteBatch.Begin();
            if (!searchExpired)
            {
                game.spriteBatch.Draw(titleTex, Vector2.Zero, Color.White);

                Vector2 offset = game.menuFont.MeasureString("Searching for Host");
                Vector2 location = new Vector2(512 - (offset.X / 2), 500);
                location.X += (float)(Math.Sin(gameTime.TotalGameTime.TotalSeconds / 3) * 15);
                location.Y += (float)Math.Cos(gameTime.TotalGameTime.TotalSeconds / 3) * 20;
                game.spriteBatch.DrawString(game.menuFont, "Searching for Host", location, Color.Red);
            }
            else
            {
                if (!allConnected)
                {
                    game.spriteBatch.DrawString(game.menuFont, "Connect SAFETY", new Vector2(300,300), Color.White);
                }
                else
                {
                    if (DreadMill.IsGameServer()) // Player 1
                    {
                        switch (player1Accepted)
                        {
                            case 1:
                                game.spriteBatch.Draw(warning2, Vector2.Zero, Color.White);

                                break;
                            case 2:
                                game.spriteBatch.Draw(warning2, Vector2.Zero, Color.White);
                                game.spriteBatch.DrawString(game.menuFont, "Waiting for Player 2", Vector2.Zero, Color.Red);
                                break;
                            default:
                                game.spriteBatch.Draw(warning1, Vector2.Zero, Color.White);
                                break;
                        }

                    }
                    else // Player 2
                    {
                        switch (player2Accepted)
                        {
                            case 1:
                                game.spriteBatch.Draw(warning2, Vector2.Zero, Color.White);

                                break;
                            case 2:
                                game.spriteBatch.Draw(warning2, Vector2.Zero, Color.White);
                                game.spriteBatch.DrawString(game.menuFont, "Waiting for Player 1", Vector2.Zero, Color.Red);
                                break;
                            default:
                                game.spriteBatch.Draw(warning1, Vector2.Zero, Color.White);
                                break;
                        }
                    }
                }
            }
            game.spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}