using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;


namespace Dreadmill
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class PlayState : Microsoft.Xna.Framework.DrawableGameComponent
    {
        Game1 game;
        GraphicsDeviceManager graphics;
        ContentManager content;
        GameComponentCollection components;

        KeyboardState oldkb;

        SpriteBatch spriteBatch;

        Texture2D sidewalk, road;

        public Player player1, player2;

        List<Zombie> zombies;

        List<PowerUp> powerUps;

        Random rand;

        float roadScroll = 0f;
        DateTime gameStart;
        bool running = false;

        Rectangle roadRectangle, sidewalkLeft, sidewalkRight;

        public PlayState(Game1 game, GraphicsDeviceManager graphics, ContentManager content, GameComponentCollection components)
            : base(game)
        {
            // TODO: Construct any child components here
            this.graphics = graphics;
            this.content = content;
            this.components = components;
            this.game = game;

            player1 = new Player(game, graphics, content, components, "Player 1");
            player2 = new Player(game, graphics, content, components, "Player 2");

            zombies = new List<Zombie>();
            powerUps = new List<PowerUp>();

            rand = new Random();
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here
            spriteBatch = new SpriteBatch(graphics.GraphicsDevice);

            player1.Initialize();
            player2.Initialize();

            oldkb = Keyboard.GetState();

            base.Initialize();
        }

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            sidewalk = content.Load<Texture2D>("Sidewalk");
            road = content.Load<Texture2D>("Road");

            player1.LoadContent("xx");
            player2.LoadContent("zzz");

            roadRectangle = new Rectangle(0, 0, GraphicsDevice.Viewport.Width - (sidewalk.Width * 2), GraphicsDevice.Viewport.Height * 2);
            sidewalkLeft = new Rectangle(0, 0, sidewalk.Width, GraphicsDevice.Viewport.Height * 2);
            sidewalkRight = new Rectangle(0,0, sidewalk.Width, GraphicsDevice.Viewport.Height * 2);
            // TODO: use this.Content to load your game content here

            //Dreadmill Initialiser
            DreadmillDriver.Initialise();
        }

        public void InitialiseState()
        {
            player1.position.X = 800;
            player2.position.X = 200;
            gameStart = DateTime.Now;
        }

        protected override void Dispose(bool disposing)
        {
            DreadmillDriver.Shutdown();
            base.Dispose(disposing);
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            if (DreadmillNetwork.gameOver)
            {
                if (DreadmillNetwork.winner == "END1")
                game.gameOver(player1);
                else
                    game.gameOver(player2);
            }

            int newSpeed = 150 - ((int)(DateTime.Now - gameStart).TotalSeconds / 4);

            if (newSpeed != player1.treadmillSpeed)
            {
                //Force Update
                player1.treadmillSpeed = newSpeed;
                player2.treadmillSpeed = newSpeed;
                setGameSpeed(newSpeed);
            }

            if (game.rand.Next(200) < 1)
                game.soundEngine.PlayBackground("Laugh");
            // Derive Scroll Speed from treadmill
            // Invert
            float speedA = (player1.treadmillSpeed - 200) * -1;
            float speedB = (player2.treadmillSpeed - 200) * -1;
            game.roadSpeed = 1.0f + ((speedA + speedB) / 200) * 2;
            //Keyboard Debugging Trash
            if (DreadMill.IsGameServer())
            {
                UpdatePlayerPosition(player1);
                UpdateRemotePlayerPosition(player2);
            }

            else //Game Client
            {
                UpdatePlayerPosition(player2);
                UpdateRemotePlayerPosition(player1);
            }
            DreadmillNetwork.Update();
            
            roadScroll -= game.roadSpeed;
            //roadRectangle.Y -= scrollSpeed;
            if (roadScroll <= 0)
            {
                roadScroll += 128;
            }

            player1.Update(gameTime);
            player2.Update(gameTime);

            List<PowerUp> cleanUpPowerUps = new List<PowerUp>();

            foreach(PowerUp pU in powerUps)
            {
                pU.Update(gameTime);

                if(!pU.hit)
                {
                    if(boundingBoxCollision(pU.boundingBox, player1.boundingBox))
                    {
                        pU.hit = true;
                        lowerPlayer(player1);
                        cleanUpPowerUps.Add(pU);
                    }

                    if(boundingBoxCollision(pU.boundingBox, player2.boundingBox))
                    {
                        pU.hit = true;
                        lowerPlayer(player2);
                        cleanUpPowerUps.Add(pU);
                    }
                }

                if (pU.position.Y > GraphicsDevice.Viewport.Height)
                {
                    cleanUpPowerUps.Add(pU);
                }
            }

            if(cleanUpPowerUps.Count > 0)
            {
                foreach(PowerUp pU in cleanUpPowerUps)
                {
                    powerUps.Remove(pU);
                }

                cleanUpPowerUps.Clear();
            }

            List<Zombie> cleanUpZombies = new List<Zombie>();

            foreach (Zombie z in zombies)
            {
                z.Update(gameTime);

                if (!z.hit)
                {
                    if (boundingBoxCollision(z.boundingBox, player1.boundingBox))
                    {
                        //Player1 Is Affected
                        z.hit = true;
                        elevatePlayer(player1);
                        cleanUpZombies.Add(z);
                    }

                    if (boundingBoxCollision(z.boundingBox, player2.boundingBox))
                    {
                        //Player2 Is Affected
                        z.hit = true;
                        elevatePlayer(player2);
                        cleanUpZombies.Add(z);
                    }
                }

                if (z.position.Y > GraphicsDevice.Viewport.Height)
                {
                    cleanUpZombies.Add(z);
                }
            }

            if (cleanUpZombies.Count > 0)
            {
                foreach(Zombie z in cleanUpZombies)
                {
                    zombies.Remove(z);
                }

                cleanUpZombies.Clear();
            }
            if (DreadMill.IsGameServer())
            {
                float diceRoll = rand.Next(10000);

                float chance = gameTime.ElapsedGameTime.Milliseconds / diceRoll;

                if(diceRoll < 50)
                {
                    Zombie newZombie = new Zombie(game, graphics, content, components, new Vector2(rand.Next(GraphicsDevice.Viewport.Width - 154), -205));
                    DreadmillNetwork.SendCommandToClient("ZOMBIE " + newZombie.position.X + "+" + newZombie.position.Y);
                    newZombie.Initialize();
                    newZombie.LoadContent();
                    zombies.Add(newZombie);
                }

                diceRoll = rand.Next(10000);

                if (diceRoll < 25)
                {
                    PowerUp newPowerUp = new PowerUp(game, graphics, content, components, new Vector2(rand.Next(GraphicsDevice.Viewport.Width - 154), -205));
                    DreadmillNetwork.SendCommandToClient("POWER " + newPowerUp.position.X + "+" + newPowerUp.position.Y);
                    newPowerUp.Initialize();
                    newPowerUp.LoadContent();
                    powerUps.Add(newPowerUp);
                }
            }
            else //Client
            {
              //  while (DreadmillNetwork.
                while (DreadmillNetwork.items.Count > 0)
                {
                    string item = DreadmillNetwork.items.Dequeue();
                    if (item.StartsWith("ZOMBIE"))
                    {
                        string[] data = item.Split(new Char[] { ' ', '+' });
                        if (data.Length >= 3)
                        {
                            int x = Convert.ToInt32(data[1]);
                            int y = Convert.ToInt32(data[2]);
                            Zombie newZombie = new Zombie(game, graphics, content, components, new Vector2(x, y));
                            newZombie.Initialize();
                            newZombie.LoadContent();
                            zombies.Add(newZombie);
                        }

                    }
                    else // POWER V-V
                    {
                        string[] data = item.Split(new Char[] { ' ', '+' });
                        if (data.Length >= 3)
                        {
                            int x = Convert.ToInt32(data[1]);
                            int y = Convert.ToInt32(data[2]);
                            PowerUp newPowerUp = new PowerUp(game, graphics, content, components, new Vector2(x, y));
                            newPowerUp.Initialize();
                            newPowerUp.LoadContent();
                            powerUps.Add(newPowerUp);
                        }
                    }

                }
            }

            base.Update(gameTime);
        }

        public bool boundingBoxCollision(Rectangle rect1, Rectangle rect2)
        {
            if(rect1.Intersects(rect2))
            {
                return true;
            }
            return false;
        }

        void setGameSpeed(int speed)
        {
            if(DreadMill.IsGameServer())
            {
                if (speed < 50 || speed > 200)
                DreadmillDriver.SetSpeed(player1.treadmillSpeed.ToString());
                DreadmillNetwork.SendCommandToClient("PI" + player2.treadmillSpeed.ToString());
            }
        }

        void elevatePlayer(Player p)
        {
            if (DreadMill.IsGameServer())
            {
                
                p.elevation += 100;
                if(p.elevation > 960)
                {
                    if (DreadMill.IsGameServer())
                    {
                        DreadMill.StopTreadmills();
                        DreadmillNetwork.gameOver = true;
                        if (p == player1)
                        {
                            DreadmillNetwork.winner = "END1";
                            DreadmillNetwork.SendCommandToClient("END1");
                        }
                        else
                        {
                            DreadmillNetwork.winner = "END2";
                            DreadmillNetwork.SendCommandToClient("END2");
                        }
                    }
                    //GameOver Man!
                }
                if (p == player2)
                {
                    DreadmillNetwork.SendCommandToClient("SH" + player2.elevation.ToString());
                }
                else
                {
                    DreadmillDriver.SetElevation(player1.elevation.ToString());
                }
            }
        }

        void lowerPlayer(Player p)
        {
            p.elevation -= 100;
            if (p.elevation < 250)
            {
                p.elevation = 250;
            }
        }

        public override void Draw(GameTime gameTime)
        {
            if (player1.IsDown(2))
                GraphicsDevice.Clear(Color.Red);
            else if (player1.IsDown(4))
                GraphicsDevice.Clear(Color.Green);
            else if (player2.IsDown(2))
                GraphicsDevice.Clear(Color.Blue);
            else if (player2.IsDown(4))
                GraphicsDevice.Clear(Color.White);
            else
                GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend);

            SamplerState ss = new SamplerState();
            ss.AddressU = TextureAddressMode.Wrap;
            ss.AddressV = TextureAddressMode.Wrap;
            GraphicsDevice.SamplerStates[0] = ss;

            //Rectangle source = new Rectangle(0, -road.Height, GraphicsDevice.Viewport.Width, GraphicsDevice.Viewport.Height);

            spriteBatch.Draw(road, new Vector2(sidewalk.Width, 0), roadRectangle, Color.White, 0, new Vector2(0, roadScroll), 1.0f, SpriteEffects.None, 0.5f);
            spriteBatch.Draw(sidewalk, Vector2.Zero, sidewalkLeft, Color.White, 0, new Vector2(0, roadScroll), 1.0f, SpriteEffects.None, 0.5f);
            spriteBatch.Draw(sidewalk, new Vector2(GraphicsDevice.Viewport.Width - sidewalk.Width, 0), sidewalkRight, Color.White, 0, new Vector2(0, roadScroll), 1.0f, SpriteEffects.None, 0.1f);


           /* spriteBatch.DrawString(((Game1)game).debugFont, ((Game1)game).debugText + " " + player1.treadmillSpeed + " " + player2.treadmillSpeed, Vector2.Zero, Color.White);
            spriteBatch.DrawString(((Game1)game).debugFont, player1.elevation + " " + player2.elevation, new Vector2(0,20), Color.White);
            int i = 20;
            int k = Math.Max(0, DreadmillDriver.errorLog.Count - 10);
            for (int j = k; j < DreadmillDriver.errorLog.Count; j++)
            {
                i += 20;
                try
                {
                    spriteBatch.DrawString(((Game1)game).debugFont, DreadmillDriver.errorLog[j], new Vector2(0, i), Color.White);
                }
                catch (Exception ex)
                {}
            }*/

            if (DateTime.Now < gameStart.AddSeconds(5))
            {
                int seconds = 5 - ((DateTime.Now - gameStart).Seconds);
                spriteBatch.DrawString(((Game1)game).menuFont, seconds.ToString(), new Vector2(500, 300), Color.Red, 0.0f, Vector2.Zero, 4.0f, SpriteEffects.None, 0.0f);

            }
            else
            {
                if (!running)
                {
                    if (DreadMill.IsGameServer())
                    {
                        //Start the Game
                        DreadMill.StartTreadmills();
                    }
                    running = true;
                }
            }

            spriteBatch.End();


            // TODO: Add your drawing code here

            spriteBatch.Begin();

            foreach(Zombie z in zombies)
            {
                z.Draw(gameTime, spriteBatch);
            }

            foreach(PowerUp pU in powerUps)
            {
                pU.Draw(gameTime, spriteBatch);
            }

            spriteBatch.End();

            player1.Draw(gameTime);
            player2.Draw(gameTime);

            base.Draw(gameTime);
        }

        void UpdatePlayerPosition(Player player)
        {
            KeyboardState currentKeyboardState = Keyboard.GetState();

            player.SetInput(DreadmillDriver.GetInput()); //Local to Server and Client

            if (currentKeyboardState.IsKeyDown(Keys.Left) && currentKeyboardState.IsKeyDown(Keys.Right))
                player.velocity.X = 0;
            else if (currentKeyboardState.IsKeyDown(Keys.Right))
                player.velocity.X = 3;
            else if (currentKeyboardState.IsKeyDown(Keys.Left))
                player.velocity.X = -3;
            else
                player.velocity.X = 0;

            if (!player.IsDown(5))
                DreadMill.StopTreadmills();

            Vector2 kbVelocity = player.velocity;

            if (player.IsDown(1) && player.IsDown(3))
                player.velocity.X = 0;// Right
            else if (player.IsDown(1))
                player.velocity.X = 3;
            else if (player.IsDown(3)) // Left
                player.velocity.X = -3;
            else
                player.velocity.X = 0;

            if (player.velocity == Vector2.Zero)
                player.velocity = kbVelocity;

            //Update Data for sending to remote Screen
            DreadmillNetwork.local.position = player.position;
            DreadmillNetwork.local.velocity = player.velocity;

            //Server Only
            if (DreadMill.IsGameServer())
            {
                if (oldkb.IsKeyUp(Keys.G) && Keyboard.GetState().IsKeyDown(Keys.G))
                {
                    if (DreadMill.IsRunning())
                    {
                        DreadMill.StopTreadmills();
                    }
                    else
                    {
                        DreadMill.StartTreadmills();
                    }
                }

                int oldSpeed = player.treadmillSpeed;
                if (oldkb.IsKeyDown(Keys.Up))
                {
                    player.treadmillSpeed += 3;
                }
                else if (oldkb.IsKeyDown(Keys.Down))
                {
                    player.treadmillSpeed -= 3;
                }
                if (oldSpeed != player.treadmillSpeed)
                {
                    DreadmillDriver.SetSpeed(player.treadmillSpeed.ToString());
                }

                oldSpeed = player2.treadmillSpeed;
                if (oldkb.IsKeyDown(Keys.W))
                {
                    player2.treadmillSpeed += 3;
                }
                else if (oldkb.IsKeyDown(Keys.S))
                {
                    player2.treadmillSpeed -= 3;
                }
                if (oldSpeed != player2.treadmillSpeed)
                {
                    if (player2.treadmillSpeed < 50 || player2.treadmillSpeed > 200)
                        return;
                    DreadmillNetwork.SendCommandToClient("PI" + player2.treadmillSpeed.ToString());
                }
                int oldElevation = player.elevation;
                if (oldkb.IsKeyDown(Keys.E))
                {
                    player.elevation += 1;
                }
                else if (oldkb.IsKeyDown(Keys.D))
                {
                    player.elevation -= 1;
                }
                if (oldElevation != player.elevation)
                {
                    DreadmillDriver.SetElevation(player.elevation.ToString());
                }

                oldElevation = player2.elevation;
                if (oldkb.IsKeyDown(Keys.Q))
                {
                    player2.elevation += 1;
                }
                else if (oldkb.IsKeyDown(Keys.A))
                {
                    player2.elevation -= 1;
                }
                if (oldElevation != player2.elevation)
                {
                    DreadmillNetwork.SendCommandToClient("SH" + player2.elevation.ToString());
                }

            }
            oldkb = currentKeyboardState;
        }

        void UpdateRemotePlayerPosition(Player player)
        {
            player.SetInput(DreadmillNetwork.GetClientInput());

            if (!player.IsDown(5))
                DreadMill.StopTreadmills();
            //update Remote Player
            if (DreadmillNetwork.PUpdate)
            {
                player.position = DreadmillNetwork.remote.position;
                DreadmillNetwork.PUpdate = false;
            }
            if (DreadmillNetwork.VUpdate)
            {
                player.velocity = DreadmillNetwork.remote.velocity;
                DreadmillNetwork.VUpdate = false;
            }
        }   
    }
}
