using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

//Serial Port Access
using System.IO.Ports;

namespace Dreadmill
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class Player : Microsoft.Xna.Framework.DrawableGameComponent
    {
        Game game;
        GraphicsDeviceManager graphics;
        ContentManager content;
        GameComponentCollection components;

        public Texture2D texture;
        public Vector2 position;
        public Vector2 velocity;

        public int treadmillSpeed = 150;
        public int elevation = 250; //Range = 250-960

        byte input = 16;
        byte oldInput = 16;

        SpriteBatch spriteBatch;

        public Rectangle boundingBox;

        private float runSpeed = 300;

        public String name;

        public Player(Game1 game, GraphicsDeviceManager graphics, ContentManager content, GameComponentCollection components, String name)
            : base(game)
        {
            this.graphics = graphics;
            this.content = content;
            this.components = components;
            this.game = game;
            this.position = new Vector2(512 - 64, 768 - 128);
            
            runSpeed = game.rand.Next(200, 400);

            this.name = name;

            // TODO: Construct any child components here
        }

        public void SetInput(byte input)
        {
            oldInput = this.input;
            this.input = input;
        }

        public bool IsDown(int button)
        {
            int data = input >> button - 1;
            data = data & 0x01;
            if (data == 0)
                return false;
            return true;
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();

            spriteBatch = new SpriteBatch(graphics.GraphicsDevice);
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            position += velocity;
            
            if(position.X < 0)
            {
                position.X = 0;
            }
            else if (position.X > graphics.GraphicsDevice.Viewport.Width - texture.Width)
            {
                position.X = graphics.GraphicsDevice.Viewport.Width - texture.Width;
            }

            boundingBox.X = (int)position.X;
            boundingBox.Y = (int)position.Y;
            base.Update(gameTime);
        }

        public void LoadContent(String textureName)
        {
            this.texture = content.Load<Texture2D>(textureName);

            boundingBox = new Rectangle((int)this.position.X, (int)this.position.Y, texture.Width, texture.Height);
        }

        public new void Draw(GameTime gameTime)
        {
            spriteBatch.Begin();

            if (gameTime.TotalGameTime.TotalMilliseconds % runSpeed > (runSpeed / 2))
                spriteBatch.Draw(texture, boundingBox, Color.White);
            else
                spriteBatch.Draw(texture, boundingBox, null, Color.White, 0f, Vector2.Zero, SpriteEffects.FlipHorizontally, 0.1f);
            spriteBatch.End();
        }
    }
}