using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;


namespace Dreadmill
{
    public class SoundEngineXACT : Microsoft.Xna.Framework.GameComponent
    {
        // Main sound engine
        public AudioEngine audioEngine;

        // Effects
        public SoundBank backgroundEffectBank;
        public SoundBank gameplayEffectBank;
        public WaveBank soundEffectWaveBank;

        // Music (Streaming)
        public SoundBank musicBank;
        public WaveBank musicWaveBank;
        Cue currentTrack;

        Cue currentEffect;

        Cue currentBackground;

        bool playingTrack = false;

        public SoundEngineXACT(Game1 game)
            : base(game)
        {
            // TODO: Construct any child components here
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here
            audioEngine = new AudioEngine("Content\\DreadmillAudio.xgs");

            // Effects
            soundEffectWaveBank = new WaveBank(audioEngine, "Content\\Sounds.xwb");
            backgroundEffectBank = new SoundBank(audioEngine, "Content\\Background.xsb");
            gameplayEffectBank = new SoundBank(audioEngine, "Content\\Gameplay.xsb");


            // Music (Streaming) must match the XACT Project
            musicWaveBank = new WaveBank(audioEngine, "Content\\Music.xwb");
            musicBank = new SoundBank(audioEngine, "Content\\Music.xsb");

            audioEngine.Update();


            base.Initialize();
        }

        public void Play(Song song)
        {

        }

        public void PlaySong(Cue cue)
        {
            if (currentTrack != null)
                currentTrack.Stop(AudioStopOptions.AsAuthored); // Fade out if needed
            if (musicWaveBank.IsPrepared && cue.IsPrepared && cue.IsStopped)
            {
                cue.Play();
                playingTrack = true;
            }
        }

        public void PlayEffect(String effect)
        {
            currentEffect = gameplayEffectBank.GetCue(effect);
            currentEffect.Play();
        }

        public void PlayBackground(String background)
        {
            currentBackground = backgroundEffectBank.GetCue(background);
            currentBackground.Play();
        }

        public void PlaySong(String song)
        {
                currentTrack = musicBank.GetCue(song);
                currentTrack.Play();
                playingTrack = true;
        }

        public void StopMusic()
        {
            currentTrack.Stop(AudioStopOptions.AsAuthored);
            playingTrack = false;
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            if (playingTrack && currentTrack.IsStopped)
                PlaySong(currentTrack.Name);
            // Service the audio engine (Streaming cues, etc)
            audioEngine.Update();

            base.Update(gameTime);
        }
    }
}
