using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Dreadmill
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class Zombie : Microsoft.Xna.Framework.DrawableGameComponent
    {
        Game1 game;
        GraphicsDeviceManager graphics;
        ContentManager content;
        GameComponentCollection components;

        public Texture2D texture;
        public Vector2 position;

        public Rectangle boundingBox;

        float zombieSpeed = 1.0f;

        public bool hit = false;

        public Zombie(Game1 game, GraphicsDeviceManager graphics, ContentManager content, GameComponentCollection components, Vector2 position)
            : base(game)
        {
            this.graphics = graphics;
            this.content = content;
            this.components = components;
            this.game = game;
            this.position = position;

            // TODO: Construct any child components here
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
            zombieSpeed = (float)game.rand.NextDouble() + 0.5f;
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            position.Y += game.roadSpeed * zombieSpeed;

            boundingBox.X = (int)position.X;
            boundingBox.Y = (int)position.Y;

            base.Update(gameTime);
        }

        public new void LoadContent()
        {
            if (game.rand.Next(100) >=50)
                this.texture = content.Load<Texture2D>("girlZombie");
            else
                this.texture = content.Load<Texture2D>("boyZombie");
            boundingBox = new Rectangle((int)this.position.X, (int)this.position.Y, texture.Width, texture.Height);

            base.LoadContent();
        }

        public new void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            if (gameTime.TotalGameTime.TotalSeconds % (zombieSpeed/2) > (zombieSpeed / 4))
                spriteBatch.Draw(texture, new Rectangle((int)position.X, (int)position.Y, texture.Width, texture.Height), Color.White);
            else
                spriteBatch.Draw(texture, new Rectangle((int)position.X, (int)position.Y,  texture.Width, texture.Height), null, Color.White, 0, Vector2.Zero, SpriteEffects.FlipHorizontally, 0.9f);

            base.Draw(gameTime);
        }
    }
}
